--[[
Copyright (c) 2008-2011, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

-- $Revision: 30 $
-- $Date: 2011-12-24 04:23:38 +0000 (Sat, 24 Dec 2011) $
if( not Broker_GarbageFu ) then
    return
end--if
------------------------------------------------------------------------------------------------------
-- Sell Items functions
------------------------------------------------------------------------------------------------------
local L = LibStub( "AceLocale-3.0" ):GetLocale( "Broker_GarbageFu" )
local dewdrop = LibStub( "Dewdrop-2.0" )
local TR = LibStub( "LibLordFarlander-TableRecycler-1.0", true )

function Broker_GarbageFu:InitSell()
    self:MakeSellAllButton()
    self:ShowSellAllButton()
end--Broker_GarbageFu:InitSell()

function Broker_GarbageFu:IsSellAllButton()
    return self.db.profile.sellallbutton
end--Broker_GarbageFu:IsSellAllButton()

function Broker_GarbageFu:ToggleSellAllButton()
    self.db.profile.sellallbutton = not self.db.profile.sellallbutton
    if( self.db.profile.sellallbutton ) then
        MerchantFrame.sellAllButton:Show()
    else
        MerchantFrame.sellAllButton:Hide()
    end
end--Broker_GarbageFu:ToggleSellAllButton()

function Broker_GarbageFu:IsSellOnlyGrey()
    return self.db.profile.sellonlygrey
end--Broker_GarbageFu:IsSellOnlyGrey()

function Broker_GarbageFu:ToggleSellOnlyGrey()
    self.db.profile.sellonlygrey = not self.db.profile.sellonlygrey
end--Broker_GarbageFu:ToggleSellOnlyGrey()

function Broker_GarbageFu:IsSellOnOpen()
    return self.db.profile.sellonopen
end--Broker_GarbageFu:IsSellOnOpen()

function Broker_GarbageFu:ToggleSellOnOpen()
    self.db.profile.sellonopen = not self.db.profile.sellonopen
end--Broker_GarbageFu:ToggleSellOnOpen()

function Broker_GarbageFu:IsItemSellable( item )
    if( ( ( item.qual == 0 ) or not( self:IsSellOnlyGrey() ) ) and self:IsItemDroppable( item ) ) then
        return true
    end--if
    return false
end--Broker_GarbageFu:IsItemSellable( item )

function Broker_GarbageFu:IsThereItemsToSell()
    for _, item in ipairs( self.vars.items ) do
        if( self:IsItemSellable( item ) ) then
            return true
        end--if
    end--if
    return false
end--Broker_GarbageFu:IsThereItemsToSell()

function Broker_GarbageFu:GetSellItems()
    local sellItems = TR:GetTable()
    local sellValue = 0
    local CopyTable = TR.CopyTable
    local IsItemSellable = self.IsItemSellable

    for _, item in ipairs( self.vars.items ) do
        if( IsItemSellable( self, item ) ) then
            local sellItem = TR:GetTable()
            
            CopyTable( TR, sellItem, item )
            tinsert( sellItems, sellItem )
            sellValue = sellValue + sellItem.totvalue
        end--if
    end--for
    return sellItems, sellValue
end--Broker_GarbageFu:GetSellItems()

function Broker_GarbageFu:GetSellValue()
    local sellValue = 0
    local IsItemSellable = self.IsItemSellable

    for _, item in ipairs( self.vars.items ) do
        if( IsItemSellable( self, item ) ) then
            sellValue = sellValue + item.totvalue
        end--if
    end--for
    return sellValue
end--Broker_GarbageFu:GetSellValue()


function Broker_GarbageFu:StopSelling()
    local SellItemHandle = self.vars.SellItemHandle

    if( SellItemHandle ) then
        self:CancelTimer( SellItemHandle )
        SellItemHandle = nil
    end--if
end--Broker_GarbageFu:StopSelling()


function Broker_GarbageFu:SellAllItems()
    local vars = self.vars
    
    if( not vars.SellItemHandle ) then
        if( vars.sellitems ) then
            TR:ReturnTable( vars.sellitems )
        end--if
        vars.sellitems, vars.sellvalue = self:GetSellItems()
        if( vars.sellitems[1] ) then
            vars.SellItemHandle = self:ScheduleRepeatingTimer( self.SellItemEvent, 0.5, self )
        end--if
    end--if
end--Broker_GarbageFu:SellAllItems()

function Broker_GarbageFu:SellItemEvent()
    local item = self.vars.sellitems[1]

    if( item and MerchantFrame:IsVisible() ) then
        if( self:IsSafeToDelete() ) then
            if( not self:CheckItem( item ) ) then
                self:StopSelling()
                self:MyPrint( "Error detected, aborting." )
                MerchantFrame.sellAllButton:Disable()
                return
            end--if
            self:SetSafeToDelete( false )
            self:MyPrint( L["Selling x%s %s worth %s"], item.stack, item.name, self:GetMoneyString( item.totvalue ) )
            --    Added by Scott update the DB with the value of the garbage item sold
            self.db.realm.garbageprofit[UnitName( "player" )] = self.db.realm.garbageprofit[UnitName( "player" )] + item.totvalue
            UseContainerItem( item.bag, item.slot )
            TR:ReturnTable( table.remove( self.vars.sellitems, 1 ) )
        end--if
    else -- No more items or Merchant frame closed
        if( not item ) then
            self:MyPrint( L["All items sold."] )
            MerchantFrame.sellAllButton:Disable()
        end--if
        self:StopSelling()
        return
    end--if
end--Broker_GarbageFu:SellItemEvent()

------------------------------------------------------------------
-- Sell All Button functions
------------------------------------------------------------------

function Broker_GarbageFu:ShowSellAllButton()
    if( self:IsSellAllButton() ) then
        MerchantFrame.sellAllButton:Show()
    else
        MerchantFrame.sellAllButton:Hide()
    end--if
end--Broker_GarbageFu:ShowSellAllButton()

function Broker_GarbageFu:HideSellAllButton()
    MerchantFrame.sellAllButton:Hide()
end

function Broker_GarbageFu:HighlightSellAllButton()
    if( MerchantFrame.sellAllButton ) then
        if( self:IsThereItemsToSell() ) then
            MerchantFrame.sellAllButton:Enable()
        else
            MerchantFrame.sellAllButton:Disable()
        end--if
    end--if
end--Broker_GarbageFu:HighlightSellAllButton()

local buttonSize = 32
function Broker_GarbageFu:MakeSellAllButton()
    if( not MerchantFrame.sellAllButton ) then
        local sellAllButton = CreateFrame( "Button", "Broker_GarbageFu_SellItemButton", MerchantFrame )

        MerchantFrame.sellAllButton = sellAllButton
        sellAllButton:SetWidth( buttonSize )
        sellAllButton:SetHeight( buttonSize )

        local texture = sellAllButton:CreateTexture()

        texture:SetWidth( buttonSize * 1.6458333 )
        texture:SetHeight( buttonSize * 1.6458333 )
        texture:SetPoint( "CENTER", sellAllButton, "CENTER" )
        texture:SetTexture( [[Interface\Buttons\UI-Quickslot2]] )
        sellAllButton:SetNormalTexture( texture )

        local normalbg = sellAllButton:CreateTexture( nil, "BACKGROUND" )

        normalbg:SetTexture( [[Interface\AddOns\Broker_GarbageFu\img\icon]] )
        normalbg:SetAllPoints( sellAllButton )
        sellAllButton:SetPushedTexture( [[Interface\Buttons\UI-Quickslot-Depress]] )

        local texture = sellAllButton:CreateTexture()

        texture:SetTexture( [[Interface\Buttons\ButtonHilight-Square]] )
        texture:SetAllPoints( sellAllButton )
        sellAllButton:SetHighlightTexture( texture )

        local texture = sellAllButton:CreateTexture()

        texture:SetTexture( [[Interface\AddOns\Broker_GarbageFu\img\icon-disabled]] )
        texture:SetAllPoints( sellAllButton )
        sellAllButton:SetDisabledTexture( texture )
        if( Skinner and Skinner.applySkin ) then
            sellAllButton:SetPoint( "TOPRIGHT", MerchantFrame, "TOPRIGHT", -44, -10 )
        else
            sellAllButton:SetPoint( "TOPRIGHT", MerchantFrame, "TOPRIGHT", -44, -38 )
        end--if
        sellAllButton:SetScript( "OnClick", function()
                                                self:SellAllItems()
                                            end )
        sellAllButton:SetScript( "OnEnter", function( button )
                                               self:SellAllButtonOnEnter( button )
                                           end )
        sellAllButton:SetScript( "OnLeave", function()
                                               GameTooltip:Hide()
                                           end )

        local button_Enable = sellAllButton.Enable

        function sellAllButton:Enable()
            button_Enable( self )
            normalbg:Show()
        end--sellAllButton:Enable()

        local button_Disable = sellAllButton.Disable

        function sellAllButton:Disable()
            button_Disable( self )
            normalbg:Hide()
        end--sellAllButton:Disable()

        sellAllButton:SetFrameStrata( "Dialog" )
        MerchantFrame.sellAllButton:Disable()
    end--if
end--Broker_GarbageFu:MakeSellAllButton()

function Broker_GarbageFu:SellAllButtonOnEnter( button )
    GameTooltip:SetOwner( button, "ANCHOR_RIGHT" )
    if( self:IsSellOnlyGrey() ) then
        GameTooltip:SetText( L["Sell all grey items"] )
        GameTooltip:AddLine( L["SellsAll TooltipText2"] )
    else
        GameTooltip:SetText( L["Sell all garbage items"] )
        GameTooltip:AddLine( L["SellsAll TooltipText1"] )
    end--if

    local sellValue = self:GetSellValue()

    GameTooltip:AddLine( L["Value:"] .. " " .. self:GetMoneyString( sellValue ) )
    GameTooltip:Show()
end--Broker_GarbageFu:SellAllButtonOnEnter( button )


function Broker_GarbageFu:BuildSellMenu( level, value, inTooltip, value2 )
    if( level == 1 ) then
        if( MerchantFrame:IsVisible() and self:IsThereItemsToSell() ) then
            local sellValue = self:GetSellValue()

            GameTooltip:AddLine( L["Value:"].." "..self:GetMoneyString( sellValue ) )
            if( self:IsSellOnlyGrey() ) then
                dewdrop:AddLine( "text", L["Sell all grey items"] .. " " .. self:GetMoneyString( sellValue ),
                    "arg1", self, "func", "SellAllItems",
                    "tooltipTitle", L["Sell all grey items"], "tooltipText", L["SellsAll TooltipText2"],
                    "closeWhenClicked", true )
            else
                dewdrop:AddLine( "text", L["Sell all garbage items"] .. " " .. self:GetMoneyString( sellValue ),
                    "arg1", self, "func", "SellAllItems",
                    "tooltipTitle", L["Sell all garbage items"], "tooltipText", L["SellsAll TooltipText1"],
                    "closeWhenClicked", true )
            end--if
        end--if
    end--if
end--Broker_GarbageFu:BuildSellMenu( level, value, inTooltip, value2 )

function Broker_GarbageFu:BuildSellOptionsGroup()
    return {
        type = "group",
        handler = self,
        args = {
            sellallbutton = {
                type = "toggle",
                order = 1,
                name = L["Sell All Button on Merchant Window"],
                desc = L["SellAllButton TooltipText"],
                width = "double",
                get  = "IsItemIcon",
                set  = "IsSellAllButton",
            },
            sellonlygrey = {
                type = "toggle",
                order = 2,
                name = L["Only Autosell Grey Items"],
                desc = L["OnlyGrey TooltipText"],
                get  = "IsSellOnlyGrey",
                set  = "ToggleSellOnlyGrey",
            },
            sellonmerchantopen = {
                type = "toggle",
                order = 3,
                name = L["Sell when opening Merchant Window"],
                desc = L["SellOnOpen TooltipText"],
                width = "double",
                get  = "IsSellOnOpen",
                set  = "ToggleSellOnOpen",
            },
        },
    }
end--Broker_GarbageFu:BuildSellOptionsGroup()
